/*
 * (C) Copyright 2001 by Thomas Driemeyer <thomas@bitrot.de> www.bitrot.de.
 *
 * This is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Library General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
 * General Public License for more details.
 *
 *	Window::Window
 *	Window::make_history_combo
 *	Window::~Window
 *	Window::slot_clear
 *	Window::slot_clear_p
 *	Window::slot_clear
 *	Window::slot_mode
 *	Window::slot_next
 *	Window::slot_seed
 *	Window::slot_compute
 *	Window::compute
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <qlayout.h>
#include <qcombobox.h>
#include "gui.h"
#include "skey_md.h"
#include "skey.h"

extern bool vgascreen;			// big screen, 640x480 (Zaurus 6000)
const int vgaheight = 50;		// if big screen, button height


//----------------------------- make gui --------------------------------------
//
// initialize and create the application window and all widgets
// The ComboBoxes use Windows Style because that makes them look more like
// LineEdits. It may be in a little bit of bad taste but here is actually
// looks better.
//

Window::Window(
	Config		*c)
	: QWidget()
{
	union {QLabel *l; QPushButton *b; QComboBox *c; QLineEdit *e;} w;

	saved_phrase = 0;
	config = c;
	config->read();
	int last = config->get_lasthistory();
	const History *hist = config->get_history(last >= 0 ? last : 0);

	QGridLayout *layout = new QGridLayout(this, nrows, ncolumns, border);
	layout->setColStretch(0, 1);
	layout->setColStretch(1, 3);	// middle column is n times as wide
	layout->setColStretch(2, 1);

	QFont *bigfont, *smallfont;
	if (!vgascreen) {
		bigfont   = new QFont("helvetica", 16, QFont::Bold);
		smallfont = new QFont("helvetica", 10);
	} else {
		bigfont   = new QFont("helvetica", 28, QFont::Bold);
		smallfont = new QFont("helvetica", 16);
	}
	w.l = new QLabel("Mode:", this);			// mode
	layout->addMultiCellWidget(w.l, 0, 0, 0, 0);
	w.c = w_mode = new QComboBox(FALSE, this);
	w.c->insertItem("MD4", 0);
	w.c->insertItem("MD5", 1);
	w.c->setCurrentItem(hist && hist->md4 ? 0 : 1);
	if (vgascreen) w.c->setFixedHeight(vgaheight);
	connect(w.c, SIGNAL(activated(int)), this, SLOT(slot_mode(int)));
	layout->addMultiCellWidget(w.c, 0, 0, 1, 1);

	w.b = new QPushButton("Clear", this);			// clear
	if (vgascreen) w.b->setFixedHeight(vgaheight);
	connect(w.b, SIGNAL(clicked()), this, SLOT(slot_clear()));
	layout->addMultiCellWidget(w.e, 0, 0, 2, 2);

	w.l = new QLabel("Number:", this);			// number
	layout->addMultiCellWidget(w.l, 1, 1, 0, 0);
	w.e = w_number = new QLineEdit(this);
	if (vgascreen) w.e->setFixedHeight(vgaheight);
	if (hist && hist->number) {
		char buf[20];
		sprintf(buf, "%d", hist->number);
		w.e->setText((QString)buf);
	}
	layout->addMultiCellWidget(w.e, 1, 1, 1, 1);
	connect(w.e, SIGNAL(textChanged(const QString &)), this,
			SLOT(slot_clear(const QString &)));

	w.b = new QPushButton("Next", this);			// next
	if (vgascreen) w.b->setFixedHeight(vgaheight);
	connect(w.b, SIGNAL(clicked()), this, SLOT(slot_next()));
	layout->addMultiCellWidget(w.e, 1, 1, 2, 2);

	w.l = new QLabel("Seed:", this);			// seed
	layout->addMultiCellWidget(w.l, 2, 2, 0, 0);
	w.c = w_seed = new QComboBox(TRUE, this);
	w.c->setInsertionPolicy(w.c->NoInsertion);
	if (vgascreen) w.c->setFixedHeight(vgaheight);
	connect(w.c,  SIGNAL(activated(int)),   this, SLOT(slot_seed(int)));
	connect(this, SIGNAL(sig_clear_seed()), w.c,  SLOT(clearEdit()));
	layout->addMultiCellWidget(w.c, 2, 2, 1, 2);
	make_history_combo();

	w.l = new QLabel("Phrase:", this);			// phrase
	layout->addMultiCellWidget(w.l, 3, 3, 0, 0);
	w.e = w_phrase = new QLineEdit(this);
	if (vgascreen) w.e->setFixedHeight(vgaheight);
	layout->addMultiCellWidget(w.e, 3, 3, 1, 2);
	connect(w.e, SIGNAL(textChanged(const QString &)), this,
			SLOT(slot_clear_p(const QString &)));

	w.b = new QPushButton("Compute", this);			// compute
	if (vgascreen) w.b->setFixedHeight(vgaheight*2);
	connect(w.b, SIGNAL(clicked()), this, SLOT(slot_compute()));
	layout->addMultiCellWidget(w.b, 4, 4, 0, 2);

	w.l = w_message = new QLabel(0, this);			// message
	w.l->setFont(*smallfont);
	layout->addMultiCellWidget(w.l, 5, 5, 0, 2);

	w.l = new QLabel("Reply:", this);			// reply
	layout->addMultiCellWidget(w.l, 6, 6, 0, 0);
	w.l = w_reply_w0 = new QLabel(0, this);
	w.l->setFont(*bigfont);
	layout->addMultiCellWidget(w.l, 6, 6, 1, 2);
	w.l = w_reply_w1 = new QLabel(0, this);
	w.l->setFont(*bigfont);
	layout->addMultiCellWidget(w.l, 7, 7, 1, 2);
	w.l = new QLabel("Hex:", this);
	layout->addMultiCellWidget(w.l, 8, 8, 0, 0);
	w.l = w_reply_h = new QLabel(0, this);
	w.l->setFont(*bigfont);
	layout->addMultiCellWidget(w.l, 8, 8, 1, 2);
}


//
// put the current number/seed/phrase history into the seed history popup.
// This must be done at startup to show the saved history from the config
// file, and whenever the history is changed due to computing.
//

void Window::make_history_combo()
{
	w_seed->clear();
	for (int hi=0; const History *hist=config->get_history(hi); hi++)
		w_seed->insertItem(hist->seed, hi);
	if (int last = config->get_lasthistory())
		w_seed->setCurrentItem(last);
}


//
// shut down: write the config file that describes the current state, so
// the app can come up the same way it was left
//

Window::~Window()
{
	config->write();
}


//----------------------------- slots -----------------------------------------
//
// user has started typing, clear the replies. If the userstarted typing
// into the passphrase widget, also clear the saved passphrase so that the
// new one can replace it when the user presses Compute.
//

void Window::slot_clear()		// Clear button
{
	w_number->setText((QString)"");
	w_phrase->setText((QString)"");
	emit sig_clear_seed();
	slot_clear_p(0);
}

void Window::slot_clear_p(		// character typed into passphrase
	const QString	&text)		// entered text, not used
{
	if (saved_phrase) {
		delete[] saved_phrase;
		saved_phrase = 0;
	}
	slot_clear(text);
}

void Window::slot_clear(		// character typed into number or seed
	const QString	&text)		// entered text, not used
{
	w_reply_w0->setText((QString)"");
	w_reply_w1->setText((QString)"");
	w_reply_h ->setText((QString)"");
	w_message ->setText((QString)"");
}


//
// user chose MD4 (0) or MD5 (1) from the mode pulldown. Switch algorithm.
//

void Window::slot_mode(
	int		index)		// 0=md4, 1=md5
{
	compute();
}


//
// user pressed Next button. Decrement number and recompute.
//

void Window::slot_next()
{
	const char *number = w_number->text();
	int num = number ? atoi(number) : 0;
	char buf[20];
	sprintf(buf, "%d", num > 1 ? num-1 : 1);
	w_number->setText((QString)buf);
	compute();
}


//
// user has chosen a seed from the seed pulldown. Replace both the number
// and the seed from the history, and recompute. Note that inside this slot
// function, the widget has not yet been updated, so the seed must be passed
// to compute() explicitly or it would get the previous one.
//

void Window::slot_seed(
	int		index)		// history index, 0=oldest
{
	const History *hist = config->get_history(index);
	if (hist) {
		char buf[20];
		sprintf(buf, "%d", hist->number);
		w_mode  ->setCurrentItem(hist->md4 ? 0 : 1);
		w_number->setText((QString)(hist->number ? buf          : ""));
		w_phrase->setText((QString)(hist->phrase ? hist->phrase : ""));
		compute(hist->seed);
	}
}


//
// user has pressed Compute button.
//

void Window::slot_compute()
{
	compute();
}


//----------------------------- compute ---------------------------------------
//
// recompute replies using MD4 or MD5 hashing. This is done in reply to
// some of the user actions above. Also star out the passphrase (but keep
// it stored in saved_phrase (it wouldn't do to pick up those stars next
// time).
//

void Window::compute(
	const char	*inseed)		// use this if nz, not widget
{
	bool	   md4     = w_mode->currentItem() == 0;
	const char *number = w_number->text();
	const char *seed   = inseed ? inseed
				    : (const char *)w_seed->currentText();
	const char *phrase = saved_phrase;
	int num = number ? atoi(number) : 0;

	if (num < 1) {				// have a good number?
		w_message->setText((QString)
			"Please enter challenge number greater than 0");
		w_number->setFocus();
		return;
	}
	if (!seed || !*seed) {			// have a good seed?
		w_message->setText((QString)"Please enter challenge seed");
		w_seed->setFocus();
		return;
	}
	if (!phrase) {				// no saved passphrase,
		phrase = w_phrase->text();	// get it from gui widget
		if (!phrase || !*phrase) {
			w_message->setText((QString)
					"Please enter your passphrase");
			w_phrase->setFocus();
			return;
		}
		int len = strlen(phrase);	// save phrase from gui widget
		char *newphrase = new char[len+1];
		strcpy(newphrase, phrase);

		char asterisks[100];		// blank out gui widget
		if (len > 99) len = 99;
		memset(asterisks, '*', len);	// (this clears saved_phrase)
		asterisks[len] = 0;
		w_phrase->setText((QString)asterisks);

		saved_phrase = newphrase;	// save phrase for next time
	}
	Skey skey;				// got everything, compute
	skey.compute(md4, num, seed, saved_phrase);
	w_reply_w0->setText((QString)skey.out_word1);
	w_reply_w1->setText((QString)skey.out_word2);
	w_reply_h ->setText((QString)skey.out_hex);

	w_message->setText((QString)(
			   num < 21 ? "Warning: need to renew login soon!" :
			   num == 1 ? "WARNING: don't log in, renew login NOW!"
				    : ""));
	config->insert_history(md4, num, seed, saved_phrase);
	make_history_combo();
}
