/*
 * (C) Copyright 2001 by Thomas Driemeyer <thomas@bitrot.de> www.bitrot.de.
 *
 * This is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Library General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library
 * General Public License for more details.
 */

#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include "config.h"			// I don't like recursive includes but
					// this moc thing forces me to do this
					// because I need Config below. Stupid.

const int window_width	= 240;
const int window_height	= 320;

class Window : public QWidget
{
	Q_OBJECT
  public:
	Window(Config *);		// initialize from file and build gui
	~Window();			// shut down and write config to file

  public slots:
	void slot_mode(int);		// user has selected MD4/MD5 mode
	void slot_next();		// user has pressed Next button
	void slot_seed(int);		// user has chosen seed from history
	void slot_compute();		// user has pressed Compute button
	void slot_clear();		// user has pressed Clear button
	void slot_clear(const QString &);  // user is typing text, clear reply
	void slot_clear_p(const QString &);// user is typing phrase, clr pw/rep

  signals:
	void sig_clear_seed();		// sent to seed's clearEdit slot

  private:
	void compute(const char *s=0);	// compute hashes and redraw
	void make_history_combo();	// put history into seed combobox popup

	static const int nrows	  = 9;	// window is divided into 9*3 cells
	static const int ncolumns = 3;
	static const int border   = 8;	// space at the edge of the app window

	Config		*config;	// current mode and history

	QComboBox	*w_mode;	// mode: md4 or md5
	QLineEdit	*w_number;	// challenge number
	QComboBox	*w_seed;	// challenge seed
	QLineEdit	*w_phrase;	// user-supplied passphrase
	QLabel		*w_message;	// error messages
	QLabel		*w_reply_w0;	// first three words of reply
	QLabel		*w_reply_w1;	// next three words of reply
	QLabel		*w_reply_h;	// hex reply

	char		*saved_phrase;	// saved passphrase, or 0
};
