/*
 * this is the entry point for the MD4/MD5 computation. It is called from
 * the rest of the OTP tool. No other function is. Anything here was adapted
 * from skey-2.2, which originated at RSA. See skey_md[45].c for copyrights.
 *
 * All code in skey*.c is derived from RSA code. I changed the indentation,
 * removed code I didn't need, rewrote the calling code (everything above f),
 * collected the code into fewer files, and allowed both MD4 and MD5 in the
 * same program, but the algorithms themselves are exactly the same as before.
 * thomas@bitrot.de, 7 Jan 2001.
 */

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "skey_md.h"
#include "skey_table.h"
#include "skey.h"


void Skey::compute(
	bool		md4,		/* true for md4, false for md5 */
	int		number,		/* input number */
	const char	*seed,		/* input seed */
	const char	*phrase)	/* input passphrase */
{
	int		i, parity;
	char		key[9];
	char		buf[1100], *p;

	assert(number > 0);
	assert(strlen(seed) + strlen(phrase) + 1 < sizeof(buf));
	sprintf(buf, "%s%s", seed, phrase);
	for (p=buf; *p; p++)
		*p &= 0x7f;

	use_md4 = md4;
	f(key, buf, strlen(buf));
	while (number--)
		f(key, key, 8);

	sprintf(out_hex, "%02X%02X %02X%02X %02X%02X %02X%02X",
					key[0] & 0xff, key[1] & 0xff,
					key[2] & 0xff, key[3] & 0xff,
					key[4] & 0xff, key[5] & 0xff,
					key[6] & 0xff, key[7] & 0xff);

	for (parity=0, i=0; i < 64; i+=2)		/* compute parity */
		parity += extract(key, i, 2);
	key[8] = (char)parity << 6;

	*out_word1 = 0;
	strncat(out_word1, Wp[extract(key,  0, 11)], 4);
	strcat (out_word1, " ");
	strncat(out_word1, Wp[extract(key, 11, 11)], 4);
	strcat (out_word1, " ");
	strncat(out_word1, Wp[extract(key, 22, 11)], 4);

	*out_word2 = 0;
	strncat(out_word2, Wp[extract(key, 33, 11)], 4);
	strcat (out_word2, " ");
	strncat(out_word2, Wp[extract(key, 44, 11)], 4);
	strcat (out_word2, " ");
	strncat(out_word2, Wp[extract(key, 55, 11)], 4);
}


/*
 * The one-way function f(). Takes n in bytes and returns 8 out bytes
 */

void Skey::f(
	char *out,
	char *in,
	int n)
{
	MD_CTX md;
	unsigned long results[4];

	if (use_md4) {
		MD4Init(&md);
		MD4Update(&md, (unsigned char *)in, n);
		MD4Final((unsigned char *)results, &md);
	} else {
		MD5Init(&md);
		MD5Update(&md, (unsigned char *)in, n);
		MD5Final((unsigned char *)results, &md);
	}
	/* Fold 128 to 64 bits */
	results[0] ^= results[2];
	results[1] ^= results[3];

	/* Only works on byte-addressed little-endian machines!! */
	memcpy(out, (char *)results, 8);
}


/*
 * Extract 'length' bits from the char array 's' starting with bit 'start'
 * (Used for six-word encoding in md_compute.)
 */

unsigned long Skey::extract(
	char *s,
	int start,
	int length)
{
	unsigned char cl, cc, cr;
	unsigned long x;

	assert(length <= 11);
	assert(start >= 0);
	assert(length >= 0);
	assert(start + length <= 66);

	cl = s[start/8];
	cc = s[start/8 + 1];
	cr = s[start/8 + 2];
	x = (long)(cl<<8 | cc) <<8 | cr;
	x >>= 24 - (length + start % 8);
	x &= 0xffff >> 16-length;
	return(x);
}
